@@ -1,3 +1,4 @@
+#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h> //for memcmp()
@@ -566,3 +567,42 @@ void field_clear(field_ptr f)
     mpz_clear(f->order);
     f->field_clear(f);
 }
+
+void pbc_mpz_out_raw_n(unsigned char *data, int n, mpz_t z)
+{
+    size_t count;
+    if (mpz_sgn(z)) {
+	count = (mpz_sizeinbase(z, 2) + 7) / 8;
+	mpz_export(&data[n - count], NULL, 1, 1, 1, 0, z);
+	memset(data, 0, n - count);
+    } else {
+	memset(data, 0, n);
+    }
+}
+
+void pbc_mpz_from_hash(mpz_t z, mpz_t limit,
+	unsigned char *data, unsigned int len)
+{
+    size_t i = 0, n, count = (mpz_sizeinbase(limit, 2) + 7) / 8;
+    unsigned char buf[count];
+    unsigned char counter = 0;
+    int done = 0;
+    for(;;) {
+	if (len >= count - i) {
+	    n = count - i;
+	    done = 1;
+	} else n = len;
+	memcpy(buf + i, data, n);
+	i += n;
+	if (done) break;
+	buf[i] = counter;
+	counter++;
+	i++;
+	if (i == count) break;
+    }
+    assert(i == count);
+    mpz_import(z, count, 1, 1, 1, 0, buf);
+    while (mpz_cmp(z, limit) > 0) {
+	mpz_tdiv_q_2exp(z, z, 1);
+    }
+}
