@@ -11,7 +11,6 @@
 #include "fp.h"
 #include "fieldquadratic.h"
 #include "hilbert.h"
-#include "darray.h"
 #include "mnt.h"
 #include "curve.h"
 #include "pairing.h"
@@ -22,7 +21,7 @@
 
 struct mnt_pairing_data_s {
     field_t Fq, Fqx, Fqd, Fqk;
-    curve_t Eq, Etwist, Eqk;
+    field_t Eq, Etwist;
     element_t nqrinv, nqrinv2;
     mpz_t tateexp;
     int k;
@@ -140,41 +139,42 @@ static inline void d_miller_evalfn(element_t e0,
     element_add(polymod_coeff(re_out, 0), polymod_coeff(re_out, 0), c);
 }
 
-static void cc_miller_no_denom_proj(element_t res, mpz_t q, point_t P,
+static void cc_miller_no_denom_proj(element_t res, mpz_t q, element_t P,
 	element_ptr Qx, element_ptr Qy)
 {
     int m;
     element_t v;
-    point_t Z;
+    element_t Z;
     element_t a, b, c;
-    common_curve_ptr cc = P->curve->data;
     element_t t0, t1;
     element_ptr t2 = a, t3 = b, t4 = c;
     element_t e0;
     element_t z, z2;
+    element_ptr Zx, Zy;
+    const element_ptr curve_a = curve_a_coeff(P);
+    const element_ptr Px = curve_x_coord(P);
+    const element_ptr Py = curve_y_coord(P);
 
     void proj_double(void)
     {
-	element_ptr x = Z->x;
-	element_ptr y = Z->y;
-	//t0 = 3x^2 + (cc->a) z^4
-	element_square(t0, x);
+	//t0 = 3x^2 + (curve_a) z^4
+	element_square(t0, Zx);
 	//element_mul_si(t0, t0, 3);
 	element_double(t1, t0);
 	element_add(t0, t0, t1);
 	element_square(t1, z2);
-	element_mul(t1, t1, cc->a);
+	element_mul(t1, t1, curve_a);
 	element_add(t0, t0, t1);
 
 	//z_out = 2 y z
-	element_mul(z, y, z);
+	element_mul(z, Zy, z);
 	//element_mul_si(z, z, 2);
 	element_double(z, z);
 	element_square(z2, z);
 
 	//t1 = 4 x y^2
-	element_square(t2, y);
-	element_mul(t1, x, t2);
+	element_square(t2, Zy);
+	element_mul(t1, Zx, t2);
 	//element_mul_si(t1, t1, 4);
 	element_double(t1, t1);
 	element_double(t1, t1);
@@ -182,8 +182,8 @@ static void cc_miller_no_denom_proj(element_t res, mpz_t q, point_t P,
 	//x_out = t0^2 - 2 t1
 	//element_mul_si(t3, t1, 2);
 	element_double(t3, t1);
-	element_square(x, t0);
-	element_sub(x, x, t3);
+	element_square(Zx, t0);
+	element_sub(Zx, Zx, t3);
 
 	//t2 = 8y^4
 	element_square(t2, t2);
@@ -193,72 +193,64 @@ static void cc_miller_no_denom_proj(element_t res, mpz_t q, point_t P,
 	element_double(t2, t2);
 
 	//y_out = t0(t1 - x_out) - t2
-	element_sub(t1, t1, x);
+	element_sub(t1, t1, Zx);
 	element_mul(t0, t0, t1);
-	element_sub(y, t0, t2);
+	element_sub(Zy, t0, t2);
     }
 
     void proj_mixin(void)
     {
-	element_ptr Px = Z->x;
-	element_ptr Py = Z->y;
-	element_ptr Bx = P->x;
-	element_ptr By = P->y;
+	//t2 = Px z^2
+	element_mul(t2, z2, Px);
 
-	//t2 = Bx z^2
-	element_mul(t2, z2, Bx);
+	//t3 = Zx - t2
+	element_sub(t3, Zx, t2);
 
-	//t3 = Px - t2
-	element_sub(t3, Px, t2);
-
-	//t0 = By z^3
-	element_mul(t0, z2, By);
+	//t0 = Py z^3
+	element_mul(t0, z2, Py);
 	element_mul(t0, t0, z);
 
-	//t1 = Py - t0
-	element_sub(t1, Py, t0);
+	//t1 = Zy - t0
+	element_sub(t1, Zy, t0);
 
-	//e7 = Px + t2, use t2 to double for e7
-	element_add(t2, Px, t2);
+	//e7 = Zx + t2, use t2 to double for e7
+	element_add(t2, Zx, t2);
 
-	//e8 = Py + t0, use t0 to double for e8
-	element_add(t0, Py, t0);
+	//e8 = Zy + t0, use t0 to double for e8
+	element_add(t0, Zy, t0);
 
 	//z = z t3
 	element_mul(z, z, t3);
 	element_square(z2, z);
 
-	//Px = t1^2 - e7 t3^2
+	//Zx = t1^2 - e7 t3^2
 	//t3 now holds t3^3,
 	//t4 holds e7 t3^2
 	element_square(t4, t3);
 	element_mul(t3, t4, t3);
-	element_square(Px, t1);
+	element_square(Zx, t1);
 	element_mul(t4, t2, t4);
-	element_sub(Px, Px, t4);
+	element_sub(Zx, Zx, t4);
 
-	//t4 = e7 t3^2 - 2 Px
-	element_sub(t4, t4, Px);
-	element_sub(t4, t4, Px);
+	//t4 = e7 t3^2 - 2 Zx
+	element_sub(t4, t4, Zx);
+	element_sub(t4, t4, Zx);
 
-	//Py = (t4 t1 - e8 t3^3)/2
+	//Zy = (t4 t1 - e8 t3^3)/2
 	element_mul(t4, t4, t1);
 	element_mul(t0, t0, t3);
 	element_sub(t4, t4, t0);
-	element_halve(Py, t4);
+	element_halve(Zy, t4);
     }
 
     void do_tangent(void)
     {
-	element_ptr Zx = Z->x;
-	element_ptr Zy = Z->y;
-
 	//a = -(3x^2 + cca z^4)
 	//b = 2 y z^3
 	//c = -(2 y^2 + x a)
 	//a = z^2 a
 	element_square(a, z2);
-	element_mul(a, a, cc->a);
+	element_mul(a, a, curve_a);
 	element_square(b, Zx);
 	//element_mul_si(b, b, 3);
 	element_double(t0, b);
@@ -283,35 +275,30 @@ static void cc_miller_no_denom_proj(element_t res, mpz_t q, point_t P,
 
     void do_line(void)
     {
-	element_ptr Ax = Z->x;
-	element_ptr Ay = Z->y;
-	element_ptr Bx = P->x;
-	element_ptr By = P->y;
+	//a = -(Py z^3 - Zy)
+	//b = Px z^3 - Zx z
+	//c = Zx z Py - Zy Px;
 
-	//a = -(By z^3 - Ay)
-	//b = Bx z^3 - Ax z
-	//c = Ax z By - Ay Bx;
-
-	element_mul(t0, Ax, z);
+	element_mul(t0, Zx, z);
 	element_mul(t1, z2, z);
 
-	element_mul(a, By, t1);
-	element_sub(a, Ay, a);
+	element_mul(a, Py, t1);
+	element_sub(a, Zy, a);
 
-	element_mul(b, Bx, t1);
+	element_mul(b, Px, t1);
 	element_sub(b, b, t0);
 
-	element_mul(t0, t0, By);
-	element_mul(c, Ay, Bx);
+	element_mul(t0, t0, Py);
+	element_mul(c, Zy, Px);
 	element_sub(c, t0, c);
 
 	d_miller_evalfn(e0, a, b, c, Qx, Qy);
 	element_mul(v, v, e0);
     }
 
-    element_init(a, P->curve->field);
-    element_init(b, P->curve->field);
-    element_init(c, P->curve->field);
+    element_init(a, Px->field);
+    element_init(b, a->field);
+    element_init(c, a->field);
     element_init(t0, a->field);
     element_init(t1, a->field);
     element_init(e0, res->field);
@@ -321,9 +308,11 @@ static void cc_miller_no_denom_proj(element_t res, mpz_t q, point_t P,
     element_set1(z2);
 
     element_init(v, res->field);
-    point_init(Z, P->curve);
+    element_init(Z, P->field);
 
-    point_set(Z, P);
+    element_set(Z, P);
+    Zx = curve_x_coord(Z);
+    Zy = curve_x_coord(Z);
 
     element_set1(v);
     m = mpz_sizeinbase(q, 2) - 2;
@@ -343,7 +332,7 @@ static void cc_miller_no_denom_proj(element_t res, mpz_t q, point_t P,
     element_set(res, v);
 
     element_clear(v);
-    point_clear(Z);
+    element_clear(Z);
     element_clear(a);
     element_clear(b);
     element_clear(c);
@@ -354,17 +343,19 @@ static void cc_miller_no_denom_proj(element_t res, mpz_t q, point_t P,
     element_clear(z2);
 }
 
-static void cc_miller_no_denom_affine(element_t res, mpz_t q, point_t P,
+static void cc_miller_no_denom_affine(element_t res, mpz_t q, element_t P,
 	element_ptr Qx, element_ptr Qy)
 {
     int m;
     element_t v;
-    point_t Z;
+    element_t Z;
     element_t a, b, c;
-    const common_curve_ptr cc = P->curve->data;
-    const element_ptr cca = cc->a;
     element_t t0;
     element_t e0;
+    const element_ptr cca = curve_a_coeff(P);
+    const element_ptr Px = curve_x_coord(P);
+    const element_ptr Py = curve_y_coord(P);
+    element_ptr Zx, Zy;
 
     /* TODO: when exactly is this not needed?
     void do_vertical(void)
@@ -380,8 +371,6 @@ static void cc_miller_no_denom_affine(element_t res, mpz_t q, point_t P,
 	//a = -(3 Zx^2 + cc->a)
 	//b = 2 * Zy
 	//c = -(2 Zy^2 + a Zx);
-	const element_ptr Zx = Z->x;
-	const element_ptr Zy = Z->y;
 
 	element_square(a, Zx);
 	element_mul_si(a, a, 3);
@@ -406,15 +395,10 @@ static void cc_miller_no_denom_affine(element_t res, mpz_t q, point_t P,
 	//c = -(A.y + a * A.x);
 	//but we'll multiply by B.x - A.x to avoid division
 
-	const element_ptr Ax = Z->x;
-	const element_ptr Ay = Z->y;
-	const element_ptr Bx = P->x;
-	const element_ptr By = P->y;
-
-	element_sub(b, Bx, Ax);
-	element_sub(a, Ay, By);
-	element_mul(t0, b, Ay);
-	element_mul(c, a, Ax);
+	element_sub(b, Px, Zx);
+	element_sub(a, Zy, Py);
+	element_mul(t0, b, Zy);
+	element_mul(c, a, Zx);
 	element_add(c, c, t0);
 	element_neg(c, c);
 
@@ -422,16 +406,18 @@ static void cc_miller_no_denom_affine(element_t res, mpz_t q, point_t P,
 	element_mul(v, v, e0);
     }
 
-    element_init(a, P->curve->field);
-    element_init(b, P->curve->field);
-    element_init(c, P->curve->field);
-    element_init(t0, P->curve->field);
+    element_init(a, Px->field);
+    element_init(b, a->field);
+    element_init(c, a->field);
+    element_init(t0, a->field);
     element_init(e0, res->field);
 
     element_init(v, res->field);
-    point_init(Z, P->curve);
+    element_init(Z, P->field);
 
-    point_set(Z, P);
+    element_set(Z, P);
+    Zx = curve_x_coord(Z);
+    Zy = curve_y_coord(Z);
 
     element_set1(v);
     m = mpz_sizeinbase(q, 2) - 2;
@@ -441,10 +427,10 @@ static void cc_miller_no_denom_affine(element_t res, mpz_t q, point_t P,
 
 	if (!m) break;
 
-	point_double(Z, Z);
+	element_double(Z, Z);
 	if (mpz_tstbit(q, m)) {
 	    do_line();
-	    point_add(Z, Z, P);
+	    element_add(Z, Z, P);
 	}
 	m--;
 	element_square(v, v);
@@ -453,7 +439,7 @@ static void cc_miller_no_denom_affine(element_t res, mpz_t q, point_t P,
     element_set(res, v);
 
     element_clear(v);
-    point_clear(Z);
+    element_clear(Z);
     element_clear(a);
     element_clear(b);
     element_clear(c);
@@ -512,13 +498,13 @@ static void cc_tatepower(element_ptr out, element_ptr in, pairing_t pairing)
     }
 }
 
-static void (*cc_miller_no_denom_fn)(element_t res, mpz_t q, point_t P,
+static void (*cc_miller_no_denom_fn)(element_t res, mpz_t q, element_t P,
 	element_ptr Qx, element_ptr Qy);
 
 static void cc_pairing(element_ptr out, element_ptr in1, element_ptr in2,
 	pairing_t pairing)
 {
-    point_ptr Qbase = in2->data;
+    element_ptr Qbase = in2;
     element_t Qx, Qy;
     mnt_pairing_data_ptr p = pairing->data;
 
@@ -526,10 +512,10 @@ static void cc_pairing(element_ptr out, element_ptr in1, element_ptr in2,
     element_init(Qy, p->Fqd);
     //map from twist: (x, y) --> (v^-1 x, v^-(3/2) y)
     //where v is the quadratic nonresidue used to construct the twist
-    element_mul(Qx, Qbase->x, p->nqrinv);
+    element_mul(Qx, curve_x_coord(Qbase), p->nqrinv);
     //v^-3/2 = v^-2 * v^1/2
-    element_mul(Qy, Qbase->y, p->nqrinv2);
-    cc_miller_no_denom_fn(out, pairing->r, in1->data, Qx, Qy);
+    element_mul(Qy, curve_y_coord(Qbase), p->nqrinv2);
+    cc_miller_no_denom_fn(out, pairing->r, in1, Qx, Qy);
     cc_tatepower(out, out, pairing);
     element_clear(Qx);
     element_clear(Qy);
@@ -553,15 +539,13 @@ static int cc_is_almost_coddh(element_ptr a, element_ptr b,
     element_init(t0, pairing->GT);
     element_init(t1, pairing->GT);
     element_init(t2, pairing->GT);
-    point_ptr Pc = c->data;
-    point_ptr Pd = d->data;
     //map from twist: (x, y) --> (v^-1 x, v^-(3/2) y)
     //where v is the quadratic nonresidue used to construct the twist
-    element_mul(cx, Pc->x, p->nqrinv);
-    element_mul(dx, Pd->x, p->nqrinv);
+    element_mul(cx, curve_x_coord(c), p->nqrinv);
+    element_mul(dx, curve_x_coord(d), p->nqrinv);
     //v^-3/2 = v^-2 * v^1/2
-    element_mul(cy, Pc->y, p->nqrinv2);
-    element_mul(dy, Pd->y, p->nqrinv2);
+    element_mul(cy, curve_y_coord(c), p->nqrinv2);
+    element_mul(dy, curve_y_coord(d), p->nqrinv2);
 
     cc_miller_no_denom_fn(t0, pairing->r, a->data, dx, dy);
     cc_miller_no_denom_fn(t1, pairing->r, b->data, cx, cy);
@@ -589,23 +573,19 @@ static int cc_is_almost_coddh(element_ptr a, element_ptr b,
     return res;
 }
 
-static void Fq_to_Fqd_to_Fqk(element_t out, element_t in)
-{
-    element_field_to_polymod(fi_re(out), in);
-    element_set0(fi_im(out));
-}
-
 static void trace(element_t out, element_t in, pairing_ptr pairing)
 {
+    UNUSED_VAR(out); UNUSED_VAR(in); UNUSED_VAR(pairing);
+    /* TODO: broken
     int i;
-    point_ptr p = in->data;
-    point_t r;
-    point_t q;
-    point_ptr outp = out->data;
+    element_ptr p = in;
+    element_t r;
+    element_t q;
+    element_ptr outp = out;
     mnt_pairing_data_ptr pdp = pairing->data;
 
-    point_init(q, pdp->Eqk);
-    point_init(r, pdp->Eqk);
+    element_init(q, pdp->Eqk);
+    element_init(r, pdp->Eqk);
 
     q->inf_flag = 0;
     //map from twist: (x, y) --> (v^-1 x, v^-(3/2) y)
@@ -640,6 +620,7 @@ static void trace(element_t out, element_t in, pairing_ptr pairing)
     element_set(outp->y, polymod_coeff(fi_re(r->y), 0));
     //may have to multiply to ensure order is correct
     point_clear(r);
+    */
 }
 
 static void d_pairing_option_set(pairing_t pairing, char *key, char *value)
@@ -664,9 +645,10 @@ typedef struct pp_coeff_s *pp_coeff_ptr;
 
 static void d_pairing_pp_init(pairing_pp_t p, element_ptr in1, pairing_t pairing)
 {
-    point_ptr P = in1->data;
-    const common_curve_ptr cc = P->curve->data;
-    point_t Z;
+    element_ptr P = in1;
+    const element_ptr Px = curve_x_coord(P);
+    const element_ptr Py = curve_y_coord(P);
+    element_t Z;
     int m;
     mnt_pairing_data_ptr info = pairing->data;
     element_t t0;
@@ -675,6 +657,9 @@ static void d_pairing_pp_init(pairing_pp_t p, element_ptr in1, pairing_t pairing
     pp_coeff_t *coeff;
     mpz_ptr q = pairing->r;
     pp_coeff_ptr pp;
+    const element_ptr cca = curve_a_coeff(P);
+    element_ptr Zx;
+    element_ptr Zy;
 
     void store_abc(void)
     {
@@ -699,13 +684,11 @@ static void d_pairing_pp_init(pairing_pp_t p, element_ptr in1, pairing_t pairing
 	//a = -(3 Zx^2 + cc->a)
 	//b = 2 * Zy
 	//c = -(2 Zy^2 + a Zx);
-	const element_ptr Zx = Z->x;
-	const element_ptr Zy = Z->y;
 
 	element_square(a, Zx);
 	element_double(t0, a);
 	element_add(a, a, t0);
-	element_add(a, a, cc->a);
+	element_add(a, a, cca);
 	element_neg(a, a);
 
 	element_add(b, Zy, Zy);
@@ -725,23 +708,20 @@ static void d_pairing_pp_init(pairing_pp_t p, element_ptr in1, pairing_t pairing
 	//c = -(A.y + a * A.x);
 	//but we'll multiply by B.x - A.x to avoid division
 
-	const element_ptr Ax = Z->x;
-	const element_ptr Ay = Z->y;
-	const element_ptr Bx = P->x;
-	const element_ptr By = P->y;
-
-	element_sub(b, Bx, Ax);
-	element_sub(a, Ay, By);
-	element_mul(t0, b, Ay);
-	element_mul(c, a, Ax);
+	element_sub(b, Px, Zx);
+	element_sub(a, Zy, Py);
+	element_mul(t0, b, Zy);
+	element_mul(c, a, Zx);
 	element_add(c, c, t0);
 	element_neg(c, c);
 
 	store_abc();
     }
 
-    point_init(Z, P->curve);
-    point_set(Z, P);
+    element_init(Z, P->field);
+    element_set(Z, P);
+    Zx = curve_x_coord(Z);
+    Zy = curve_y_coord(Z);
 
     element_init(t0, Fq);
     element_init(a, Fq);
@@ -758,10 +738,10 @@ static void d_pairing_pp_init(pairing_pp_t p, element_ptr in1, pairing_t pairing
 
 	if (!m) break;
 
-	point_double(Z, Z);
+	element_double(Z, Z);
 	if (mpz_tstbit(q, m)) {
 	    do_line();
-	    point_add(Z, Z, P);
+	    element_add(Z, Z, P);
 	}
 	m--;
     }
@@ -770,7 +750,7 @@ static void d_pairing_pp_init(pairing_pp_t p, element_ptr in1, pairing_t pairing
     element_clear(a);
     element_clear(b);
     element_clear(c);
-    point_clear(Z);
+    element_clear(Z);
 }
 
 static void d_pairing_pp_clear(pairing_pp_t p)
@@ -797,7 +777,7 @@ static void d_pairing_pp_apply(element_ptr out, element_ptr in2, pairing_pp_t p)
     int m = mpz_sizeinbase(q, 2) - 2;
     pp_coeff_t *coeff = (pp_coeff_t *) p->data;
     pp_coeff_ptr pp = coeff[0];
-    point_ptr Qbase = in2->data;
+    element_ptr Qbase = in2;
     element_t e0;
     element_t Qx, Qy;
     element_t v;
@@ -808,9 +788,9 @@ static void d_pairing_pp_apply(element_ptr out, element_ptr in2, pairing_pp_t p)
 
     //map from twist: (x, y) --> (v^-1 x, v^-(3/2) y)
     //where v is the quadratic nonresidue used to construct the twist
-    element_mul(Qx, Qbase->x, info->nqrinv);
+    element_mul(Qx, curve_x_coord(Qbase), info->nqrinv);
     //v^-3/2 = v^-2 * v^1/2
-    element_mul(Qy, Qbase->y, info->nqrinv2);
+    element_mul(Qy, curve_y_coord(Qbase), info->nqrinv2);
 
     element_set1(out);
     for(;;) {
@@ -847,9 +827,8 @@ void d_pairing_clear(pairing_t pairing)
 	free(p->xpowq);
     }
 
-    curve_clear(p->Eqk);
-    curve_clear(p->Etwist);
-    curve_clear(p->Eq);
+    field_clear(p->Etwist);
+    field_clear(p->Eq);
     element_clear(p->nqrinv);
     element_clear(p->nqrinv2);
     field_clear(p->Fqk);
@@ -863,7 +842,6 @@ void pairing_init_d_param(pairing_t pairing, d_param_t param)
     mnt_pairing_data_ptr p;
     element_t a, b;
     element_t irred;
-    mpz_t one;
     int d = param->k / 2;
     int i;
 
@@ -884,7 +862,7 @@ void pairing_init_d_param(pairing_t pairing, d_param_t param)
     element_init(b, p->Fq);
     element_set_mpz(a, param->a);
     element_set_mpz(b, param->b);
-    curve_init_cc_ab(p->Eq, a, b);
+    field_init_curve_ab(p->Eq, a, b, pairing->r, param->h);
 
     field_init_poly(p->Fqx, p->Fq);
     element_init(irred, p->Fqx);
@@ -929,22 +907,18 @@ void pairing_init_d_param(pairing_t pairing, d_param_t param)
 	mpz_divexact(p->tateexp, p->tateexp, pairing->r);
     }
 
-    cc_init_map_curve(p->Eqk, p->Eq, p->Fqk, Fq_to_Fqd_to_Fqk);
-    cc_init_map_curve(p->Etwist, p->Eq, p->Fqd, element_field_to_polymod);
+    field_init_curve_ab_map(p->Etwist, p->Eq, element_field_to_polymod, p->Fqd, pairing->r, NULL);
+
     twist_curve(p->Etwist);
+
     element_init(p->nqrinv, p->Fqd);
     element_invert(p->nqrinv, field_get_nqr(p->Fqd));
     element_init(p->nqrinv2, p->Fqd);
     element_square(p->nqrinv2, p->nqrinv);
 
-    pairing->G1 = malloc(sizeof(field_t));
-    pairing->G2 = malloc(sizeof(field_t));
+    pairing->G1 = p->Eq;
+    pairing->G2 = p->Etwist;
 
-    field_init_curve_group(pairing->G1, p->Eq, pairing->r, param->h);
-    mpz_init(one);
-    mpz_set_si(one, 1);
-    field_init_curve_group(pairing->G2, p->Etwist, pairing->r, one);
-    mpz_clear(one);
     p->k = param->k;
     pairing->GT = p->Fqk;
     pairing->phi = trace;
@@ -970,7 +944,7 @@ static void compute_cm_curve(d_param_ptr param, cm_info_ptr cm)
     element_t hp, root;
     field_t fp, fpx;
     int i, n;
-    curve_t cc;
+    field_t cc;
 
     field_init_fp(fp, cm->q);
     field_init_poly(fpx, fp);
@@ -1004,33 +978,31 @@ static void compute_cm_curve(d_param_ptr param, cm_info_ptr cm)
     field_clear(fpx);
 
     //the root is the j-invariant of our desired curve
-    curve_init_cc_j(cc, root);
+    field_init_curve_j(cc, root, cm->n, NULL);
     element_clear(root);
 
     //we may need to twist it however
     {
-	point_t P;
+	element_t P;
 
 	//pick a random point P and see if it has the right order
-	point_init(P, cc);
-	point_random(P);
-	point_mul(P, cm->n, P);
-	//printf("P = ");
-	//point_out_str(stdout, 0, P);
-	//printf("\n");
+	element_init(P, cc);
+	element_random(P);
+	element_mul_mpz(P, P, cm->n);
+	//element_printf("P = %B", P);
 	//if not, we twist the curve
-	if (!point_is_inf(P)) {
+	if (!element_is0(P)) {
 	    twist_curve(cc);
 	}
-	point_clear(P);
+	element_clear(P);
     }
 
     mpz_set(param->q, cm->q);
     mpz_set(param->n, cm->n);
     mpz_set(param->h, cm->h);
     mpz_set(param->r, cm->r);
-    element_to_mpz(param->a, ((common_curve_ptr) cc->data)->a);
-    element_to_mpz(param->b, ((common_curve_ptr) cc->data)->b);
+    element_to_mpz(param->a, curve_field_a_coeff(cc));
+    element_to_mpz(param->b, curve_field_b_coeff(cc));
     param->k = cm->k;
     {
 	mpz_t z;
@@ -1047,7 +1019,7 @@ static void compute_cm_curve(d_param_ptr param, cm_info_ptr cm)
 	mpz_divexact(param->hk, param->nk, z);
 	mpz_clear(z);
     }
-    curve_clear(cc);
+    field_clear(cc);
     field_clear(fp);
 }
 
